#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# 此文件为模型预测的脚本文件

import numpy as np
import os
import paddle

from reader import test_data_loader
from multinms import multiclass_nms
from yolov3 import YOLOv3
from draw_results import draw_results

import argparse

def parse_args():
    parser = argparse.ArgumentParser("Evaluation Parameters")
    parser.add_argument(
        '--image_dir',
        type=str,
        default='./dataset/test/images',
        help='the directory of test images')
    parser.add_argument(
        '--weight_file',
        type=str,
        default='./predict_input/param.pdparams',
        help='the path of model parameters')
    parser.add_argument("--output_dir", type=str, default='./predict_output')
    args = parser.parse_args()
    return args

args = parse_args()

# Op的输入输出的存储路径，可以通过环境变量获取，环境变量名为输入输出的名称，即赋值时所用的dict的key
# 具体讲解可参考：https://cloud.baidu-int.com/icloud/PaddleCloud/%E6%93%8D%E4%BD%9C%E6%8C%87%E5%8D%97/pipeline%E6%93%8D%E4%BD%9C%E6%8C%87%E5%8D%97/%E7%BC%96%E6%8E%92/%E8%AF%A6%E8%A7%A3#%E9%80%9A%E8%BF%87%E7%8E%AF%E5%A2%83%E5%8F%98%E9%87%8F%E8%8E%B7%E5%8F%96
WEIGHT_FILE = os.getenv("model") + "/param.pdparams"


ANCHORS = [10, 13, 16, 30, 33, 23, 30, 61, 62, 45, 59, 119, 116, 90, 156, 198, 373, 326]

ANCHOR_MASKS = [[6, 7, 8], [3, 4, 5], [0, 1, 2]]

VALID_THRESH = 0.01

NMS_TOPK = 400
NMS_POSK = 100
NMS_THRESH = 0.45

NUM_CLASSES = 7


if __name__ == '__main__':
    model = YOLOv3(num_classes=NUM_CLASSES)
    model_state_dict = paddle.load(WEIGHT_FILE)
    model.load_dict(model_state_dict)
    model.eval()

    total_results = []
    test_loader = test_data_loader(args.image_dir, batch_size= 1)
    print("start processing pics...")
    for i, data in enumerate(test_loader()):
        img_name, img_data, img_scale_data = data
        img = paddle.to_tensor(img_data)
        img_scale = paddle.to_tensor(img_scale_data)

        outputs = model.forward(img)
        bboxes, scores = model.get_pred(outputs,
                                 im_shape=img_scale,
                                 anchors=ANCHORS,
                                 anchor_masks=ANCHOR_MASKS,
                                 valid_thresh = VALID_THRESH)

        bboxes_data = bboxes.numpy()
        scores_data = scores.numpy()
        results = multiclass_nms(bboxes_data, scores_data,
                      score_thresh=VALID_THRESH, 
                      nms_thresh=NMS_THRESH, 
                      pre_nms_topk=NMS_TOPK, 
                      pos_nms_topk=NMS_POSK)
        print("processed " + str(i) + " pics")
        for j in range(len(results)):
            result_j = results[j]
            img_name_j = img_name[j]
            total_results.append([img_name_j, result_j.tolist()])

if not os.path.exists(args.output_dir):
        os.makedirs(args.output_dir)

for result in total_results:
    print("saving result of : " + args.image_dir + "/" + result[0] + ".jpeg")
    draw_results(result[1], args.image_dir + "/" + result[0] + ".jpeg", args.output_dir + "/" + result[0] + ".png")

